Package: mockery
Title: Mocking Library for R
Version: 0.4.5
Authors@R: c(
    person("Noam", "Finkelstein", role = "aut"),
    person("Lukasz", "Bartnik", role = "aut"),
    person("Jim", "Hester", role = "aut"),
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre"))
  )
Description: The two main functionalities of this package are creating
    mock objects (functions) and selectively intercepting calls to a given
    function that originate in some other function. It can be used with
    any testing framework available for R. Mock objects can be injected
    with either this package's own stub() function or a similar
    with_mocked_binding() facility present in the 'testthat' package.
License: MIT + file LICENSE
URL: https://github.com/r-lib/mockery
BugReports: https://github.com/r-lib/mockery/issues
Depends: R (>= 3.6)
Imports: testthat
Suggests: knitr, R6, rmarkdown (>= 1.0)
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.2.9000
Collate: 'expectations.R' 'mockery.R' 'mock-object.R' 'stub.R'
NeedsCompilation: no
Packaged: 2025-09-03 20:07:14 UTC; hadleywickham
Author: Noam Finkelstein [aut],
  Lukasz Bartnik [aut],
  Jim Hester [aut],
  Hadley Wickham [aut, cre]
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2025-09-04 07:10:02 UTC
Built: R 4.6.0; ; 2026-04-28 17:57:50 UTC; unix
