# Generated by roxygen2: do not edit by hand

S3method(length,mock)
export(expect_args)
export(expect_call)
export(expect_called)
export(mock)
export(mock_args)
export(mock_calls)
export(stub)
importFrom(testthat,expect)
importFrom(testthat,expect_equal)
importFrom(testthat,fail)
