#!/bin/bash
# This tests https://github.com/vektra/mockery/pull/682
go run github.com/go-task/task/v3/cmd/task mocks || exit 1

FILE="pkg/fixtures/recursive_generation/subpkg_with_only_autogenerated_files/Foo_mock.go"
if ! [ -f $FILE ]; then
    echo "ERROR: $FILE does not exist"
    exit 1
fi
