#!/bin/bash
# This checks for issue #920, that fixes "include-auto-generated: false" option.

go run github.com/go-task/task/v3/cmd/task mocks || exit 1

FILE="mocks/github.com/vektra/mockery/v2/pkg/fixtures/auto_generated/AutoGenerated_mock.go"

if [ -f $FILE ]; then
    echo "ERROR: $FILE was generated, but it should be ignored"
    exit 1
fi
