PLEASE READ
-------------

DO NOT submit tickets without _first_ using the latest version of Golang, clearing your local golang package cache, and re-building mockery using the _latest_ Golang version and the _latest_ version of mockery. Please provide evidence this has been done in your issue. Failure to provide this evidence will likely result in your issue being closed.

Description
-------------

[Description of the bug or feature]

Mockery Version
--------

[version of mockery being used]

Golang Version
--------------

[version of golang being used]

```
NOTE: Please upgrade to the latest golang version before submitting tickets!
```

Installation Method
-------------------

- [ ] Binary Distribution
- [ ] Docker
- [ ] brew
- [ ] go install
- [ ] Other: [specify]

Steps to Reproduce
------------------

1. [First Step]
2. [Second Step]
3. [etc]

### Expected Behavior

[what you expect to happen]

### Actual Behavior

[what actually happened]

