---
name: Bug report
about: Create a report to help us improve
title: ''
labels: ''
assignees: ''

---

## Description

A clear and concise description of what the bug is.

## Reproducer

Steps to reproduce the behavior.

## Expected behavior

A clear and concise description of what you expected to happen.

## Mockery version

`vX.Y.Z`

## Installation Mechanism

- [] `go get`
- [] Pre-built release
- [] homebrew
- [] Other: [please describe]

## Go version

Version of Go used to build mockery from source (if applicable).
